/****************************************************************/
/*	ship.cxx defines all the methods of the ship class	*/
/*	Name : Atul Nayak					*/
/****************************************************************/
 
#include "ship.hxx"
#include "common.hxx"

// reads the coordinates of the ship from the inout file.
int ship::readInput(const char* filename)
{
	int ret = TRUE;
	int n,tmp_x,tmp_y;
	FILE* fp = fopen(filename,"r");	// open the file.
	if (!fp) {
		printf("Couldn't open file\n");
		ret = FALSE;
	}
	else {
		for (int i=0;!feof(fp);i=i+2) // store x coordinate in ith variable and y coordinate in i + 1 variable.
		{
			n=fscanf(fp,"%f %f\n", &this->vertices[i],&this->vertices[i+1]);
			ctr=ctr+2;
		}
		ret = TRUE;
	}
	return(ret);	
}

// draw the ship.
void ship::draw()
{
	// do all transformations.
	glPushMatrix();
		glColor3f(R,G,B);
		glTranslatef(this->deltaX, this->deltaY, 0);
		glScalef(scaleX,scaleY,0);
		glRotatef(this->angle,0.0,0.0,1.0);
		glBegin(GL_LINE_LOOP);
			for (int i = 0;i < this->ctr;i=i+2)
				glVertex2f(this->vertices[i], this->vertices[i+1]);
		glEnd();
	glPopMatrix();
}	

// draw an explosion.
void ship::drawExplosion()
{
	// draw concentric circles to show an explosion where the ship is located.
	int rad = 0;
	glPushMatrix();
		glColor3f(1.0,0.0,1.0);
		glTranslatef(this->deltaX, this->deltaY, 0);
		glScalef(3.0,3.0,0);
		glRotatef(this->angle,0.0,0.0,1.0);
		glBegin(GL_LINE_LOOP);
			for (int i = 0;i < 3;i++)
			{
				rad+=2;				
				for (float theta = 0;theta < (2*PI);theta+= (PI/180))
					glVertex2f(rad *cosf(theta), rad *sinf(theta));
			}	
		glEnd();
	glPopMatrix();
}	

// color the ship.
void ship::setColor(float red, float green, float blue)
{
	R = red;
	G = green;
	B = blue;
}

// scale the ship.
void ship::scale(float value)
{
	scaleX = scaleY = value;
}

// return the angle of rotation.
float ship::getAngle()
{
	return angle;
}

// reset the angle to 0 so that ship returns to it's initial position.
void ship::resetAngle()
{
	angle = 0.0;
}

// return deltaX
float ship::getDeltaX()
{
	return deltaX;
}	

// return deltaY.
float ship::getDeltaY()
{
	return deltaY;
}	

// position the ship in the centre.
void ship::resetDeltaXY()
{
	deltaX = 0;
	deltaY = 0;
}

// set the deltaX value.
void ship::SetDeltaX(float X)
{
	deltaX = X;
}
	
// set the deltaY value.
void ship::SetDeltaY(float Y)
{
	deltaY = Y;
}
	
// perform initialization.
void ship::setFirstTime()
{
	firstTime = TRUE;
	hit_by_asteroid = FALSE;
	scaleX = 1.0;
	scaleY = 1.0;
}

// reset firstTime.
void ship::resetFirstTime()
{
	firstTime = FALSE;
}

// return firstTime.
int ship::getFirstTime()
{
	return firstTime;
}
	
// set hit_by_asteroid.
void ship::setHitByAsteroid()
{
	hit_by_asteroid = TRUE;
}

// reset hit_by_asteroid.
void ship::resetHitByAsteroid()
{
	hit_by_asteroid = FALSE;
}
	
// return hit_by_asteroid.
int ship::getHitByAsteroid()
{
	return hit_by_asteroid;
}

// kill the ship.
void ship::setKilled()	
{
	killed = TRUE;
}

// revive it.
void ship::resetKilled()	
{
	killed = FALSE;
}

// retrun status whether killed or not.
int ship::getKilled()
{
	return killed;
}

// move the ship in x direction.	
void ship::moveShipX(float X)
{

	deltaX += X;
	// check if there is collision with the left or right edge.
	// if there is reverse the thrust value.
	if (((deltaX + radius) > RIGHT_EDGE) || ((deltaX - radius) < LEFT_EDGE))
		thrustX = thrustX * (-1);

}				

// move the ship in y direction.	
void ship::moveShipY(float Y)
{

	deltaY += Y;
	// check if there is collision with the bottom or top edge.
	// if there is reverse the thrust value.
	if (((deltaY + radius) > BOTTOM_EDGE) || ((deltaY - radius) < TOP_EDGE))
		thrustY = thrustY * (-1);


}				

// turn the ship ie. change the angle of rotation.
void ship::Turn(float delta)
{
	turnDir = (int) delta;
	angle = angle + delta;
}			

// calculate the thrust.	
void ship::Thrust(float maxSpeed)
{
	float increaseX = 0.5 * sinf(-angle*PI/180);
	float increaseY = 0.5 * cosf(-angle*PI/180);
		
	// check if the thrust is is between the allowable range.
	if (((thrustX + increaseX) < maxSpeed) && ((thrustX + increaseX) > -maxSpeed))	
		thrustX += increaseX;
	if (((thrustY + increaseY)< maxSpeed) && ((thrustY + increaseY) > -maxSpeed))	
		thrustY += increaseY;
}

// return thrustX.
float ship::getThrustX()
{
	return thrustX;
}

// return thrustY.
float ship::getThrustY()
{
	return thrustY;
}

// make the ship stationary. 
void ship::resetThrust()
{
	thrustX = 0;
	thrustY = 0;
}

// calculate the radius of the ship.
float ship::CalculateRadius() 
{
	// calculate the average distance of the vertice from the centre.
	float distance = 0;
	for (int i = 0;i < this->ctr; i+=2)
		distance = distance + sqrt( ((this->vertices[i]) * (this->vertices[i])) + ((this->vertices[i+1]) * (this->vertices[i+1])));
	return radius = distance/(ctr/2);
}

// return radius.
float ship::getRadius()
{
	return radius;
}

	

				

		
